=begin
  gettext.rb - GetText module

  Copyright (C) 2001-2005  Masao Mutoh
  Copyright (C) 2001-2003  Masahiro Sakai

      Masao Mutoh       <mutoh@highway.ne.jp>
      Masahiro Sakai    <s01397ms@sfc.keio.ac.jp>

  You may redistribute it and/or modify it under the same
  license terms as Ruby.

  $Id: gettext.rb,v 1.11 2006/03/15 16:34:05 mutoh Exp $
=end

require 'rbconfig'
require 'gettext/version'
require 'gettext/mo'
require 'gettext/locale'
require 'gettext/textdomain'
require 'gettext/string'

module GetText
  # If the textdomain isn't bound when calling GetText.textdomain, this error is raised.
  class NoboundTextDomainError < RuntimeError
  end

  @@__textdomain = Hash.new
  @@__textdomain_key = Hash.new

  @@__output_charset = nil

  # Bind a textdomain(%{path}/%{locale}/LC_MESSAGES/%{domainname}.mo) to your program.
  # Normally, the texdomain scope becomes a ruby-script-file. 
  # So you need to call this function each ruby-script-files. 
  # On the other hand, if you call this function under GetText::Container 
  #(gettext/container, gettext/erb, gettext/rails), the textdomain scope becomes a Class/Module.
  #
  # * domainname: the textdomain name.
  # * path: the path to the mo-files. When the value is nil, it will search default paths such as 
  #   /usr/share/locale, /usr/local/share/locale)
  # * locale: the locale string such as "ja_JP.UTF-8".  Generally, you should use GetText.set_locale instead.
  #   the value is searched order by:
  #   
  #   the value of this value > System default language.
  #   
  # * charset: output charset.  This affect the current textdomain only. Generally, you should use GetText.set_output_charset instead.
  #   the value is searched order by:
  #   
  #   the value of Locale.set_output_charset > ENV["OUTPUT_CHARSET"] > this value > System default charset.
  # * Returns: the GetText::TextDomain.
  #
  def bindtextdomain(domainname, path = nil, locale_ = nil, charset = nil)
    locale = locale_ ? Locale::Object.new(locale_) : Locale.get
    if @@__output_charset
      # Set global setting forcely.
      charset = @@__output_charset
    else
      charset = ENV["OUTPUT_CHARSET"] if ENV["OUTPUT_CHARSET"]
      @@__output_charset = charset
    end
    locale.charset = charset if charset
    Locale.set_current(locale)

    src = callersrc
    $stderr.print "bind the domain '#{domainname}' to '#{src}'. locale is #{locale.inspect}\n" if $DEBUG
    textdomain = @@__textdomain[src]
    unless textdomain
      textdomain = @@__textdomain_key[domainname]
      @@__textdomain[src] = textdomain
    end

    textdomain.set_locale(locale) if textdomain

    if ! textdomain or ! textdomain.same_property?(domainname, path, locale)
      textdomain = TextDomain.new(domainname, path, locale)
      @@__textdomain_key[domainname] = textdomain
      @@__textdomain[src] = textdomain
    end
    @@__textdomain[src]
  end

  # Binds a existed textdomain to your program. 
  # This is the same function with GetText.bindtextdomain but simpler than bindtextdomain.
  # Notice that you need to call GetText.bindtextdomain first. If the domainname hasn't bound yet, 
  # raises GetText::NoboundTextDomainError.
  # * domainname: a textdomain name.
  # * Returns: the GetText::TextDomain.
  def textdomain(domainname)
    src = callersrc
    textdomain = @@__textdomain_key[domainname]
    raise NoboundTextDomainError, "#{domainname} is not bound." unless textdomain
    textdomain.set_locale(Locale.current)
    @@__textdomain[src] = textdomain
    $stderr.print "Use the domain '#{domainname}' to '#{src}'. locale is #{textdomain.current_locale.inspect}\n" if $DEBUG
    @@__textdomain[src]
  end

  # call-seq:
  #   gettext(msgid)
  #   _(msgid)
  #
  # Translates msgid and return the message.
  # * msgid: the message id.
  # * Returns: localized text by msgid. If there are not binded mo-file, it will return msgid.
  def gettext(msgid)
    textdomain = @@__textdomain[callersrc]
    textdomain ? textdomain.gettext(msgid) : msgid
  end
  
  # call-seq:
  #   ngettext(msgid, msgid_plural, n)
  #   ngettext(msgids, n)  # msgids = [msgid, msgid_plural]
  #   n_(msgid, msgid_plural, n)
  #   n_(msgids, n)  # msgids = [msgid, msgid_plural]
  #
  # The ngettext is similar to the gettext function as it finds the message catalogs in the same way. 
  # But it takes two extra arguments for plural form.
  #
  # * msgid: the singular form.
  # * msgid_plural: the plural form.
  # * n: a number used to determine the plural form.
  # * Returns: the localized text which key is msgid_plural if n is plural(follow plural-rule) or msgid.
  #   "plural-rule" is defined in po-file.
  def ngettext(arg1, arg2, arg3 = nil)
    if arg1.kind_of?(Array)
      msgid = arg1[0]
      msgid_plural = arg1[1]
      n = arg2
    else
      msgid = arg1
      msgid_plural = arg2
      n = arg3
    end
    textdomain = @@__textdomain[callersrc]
    textdomain.ngettext(msgid, msgid_plural, n)
  end

  # This function does nothing. But it is required in order to recognize the msgid by rgettext.
  # * msgid: the message id.
  # * Returns: msgid.
  def N_(msgid)
    msgid
  end

  # This is same function as N_ but for ngettext. 
  # * msgid: the message id.
  # * msgid_plural: the plural message id.
  # * Returns: msgid.
  def Nn_(msgid, msgid_plural)
    [msgid, msgid_plural]
  end

  # call-seq:
  #   sgettext(msgid, div = '|')
  #   s_(msgid, div = '|')
  #
  # Translates msgid, but if there are no localized text, 
  # it returns a last part of msgid separeted "div".
  #
  # * msgid: the message id.
  # * div: separator or nil.
  # * Returns: the localized text by msgid. If there are no localized text, 
  #   it returns a last part of msgid separeted "div".
  # See: http://www.gnu.org/software/gettext/manual/html_mono/gettext.html#SEC151
  def sgettext(msgid, div = '|')
    textdomain = @@__textdomain[callersrc]
    if textdomain
      msg = textdomain.gettext(msgid)
    else
      msg = msgid
    end
    if msg == msgid
      if index = msg.rindex(div)
	msg = msg[(index + 1)..-1]
      end
    end
    msg
  end

  def callersrc # :nodoc:
    caller(2)[0].sub(/:\d+(?::in \`\S+\')?\Z/, '')
  end

  # Sets locale such as "de", "fr", "it", "ko", "ja_JP.eucJP", "zh_CN.EUC" ... 
  #
  # Notice that you shouldn't use this for your own Libraries.
  # * locale: a locale string or Locale::Object.
  # * src: internal usage only. You shouldn't use this.
  # * Returns: self
  def set_locale(locale, src = callersrc)
    if locale
      if locale.kind_of? Locale::Object
	ret = locale
      else
	ret = Locale::Object.new(locale.to_s)
      end
      ret.charset = @@__output_charset if @@__output_charset
    else
      ret = nil
    end
    Locale.set(ret)

    textdomain = @@__textdomain[src]
    textdomain.set_locale(ret, true) if textdomain

    self
  end

  # Same as GetText.set_locale.
  # * locale: a locale string
  # * src: internal usage only. You shouldn't use this.
  # * Returns: a locale string
  def locale=(locale)
    set_locale(locale, callersrc)
    locale
  end

  # Sets charset(String) such as "euc-jp", "sjis", "CP932", "utf-8", ... 
  # You shouldn't use this in your own Libraries.
  # * charset: an output_charset
  # * Returns: charset
  def set_output_charset(charset)
    @@__output_charset = charset
    self
  end

  # Same as GetText.set_output_charset
  # * charset: an output_charset
  # * Returns: charset
  def output_charset=(charset)
    @@__output_charset = charset
  end

  # Gets the current output_charset which is set using GetText.set_output_charset.
  # * Returns: output_charset.
  def output_charset
    @@__output_charset
  end

  # Gets the current locale.
  # * Returns: a current Locale::Object
  def locale
    Locale.current
  end

  # Sets a charset(String) such as "euc-jp", "sjis", "CP932", "utf-8", ... 
  # This affect he current textdomain only. 
  # Generaly you should use GetText.output_charset= instead. 
  # And you shouldn't use this in your own Libraries.
  # * charset: an charset
  # * src: internal usage only. You shouldn't use this.
  # * Returns: self
  def set_charset(cs, src = callersrc)
    textdomain = @@__textdomain[src]
    if textdomain
      locale = textdomain.current_locale
      locale ||= Locale.current
      locale.charset = cs if locale
      textdomain.set_locale(locale, true)
      Locale.set_current(locale)
    end
    self
  end

  # Same as GetText.set_charset.
  # * charset: an charset
  # * Returns: charset
  def charset=(cs)
    set_charset(cs, callersrc)
    cs
  end
  
  # Add default locale path.
  # * path: a new locale path. (e.g.) "/usr/share/locale/%{locale}/LC_MESSAGES/%{name}.mo"
  #    ('locale' => "ja_JP", 'name' => "textdomain")
  # * Returns: the new DEFAULT_LOCALE_PATHS
  def add_default_locale_path(path)
    TextDomain.add_default_locale_path(path)
  end

  alias :setlocale :locale= #:nodoc:
  alias :_ :gettext   #:nodoc:
  alias :n_ :ngettext #:nodoc:
  alias :s_ :sgettext #:nodoc:

  module_function :bindtextdomain, :textdomain, 
    :N_, :gettext, :_, :ngettext, :n_, 
    :sgettext, :s_, :callersrc,
    :setlocale, :set_locale, :locale=, :locale, :charset=, :set_charset,
    :set_output_charset, :output_charset=, :output_charset
end
